@@ -1,6 +1,5 @@
 import asyncio
 import collections
-import itertools
 import logging
 import random
 import types
@@ -431,9 +430,6 @@ class ConnectionsPool(AbcPool):
             self._acquiring += 1
             try:
                 conn = await self._create_new_connection(self._address)
-                # check the healthy of that connection, if
-                # something went wrong just trigger the Exception
-                await conn.execute(b"PING")
                 self._pool.appendleft(conn)
             finally:
                 self._acquiring -= 1
@@ -451,6 +447,14 @@ class ConnectionsPool(AbcPool):
                     # connection may be closed at yield point
                     await self._drop_closed()
 
+    async def _connection_post_init(self, conn: AbcConnection) -> None:
+        # check the healthy of that connection, if
+        # something went wrong just trigger the Exception
+        await conn.execute(b"PING")
+
+        if self._readonly:
+            await conn.set_readonly(self._readonly)
+
     async def _create_new_connection(self, address) -> AbcConnection:
         if self._idle_connections_collect_task is None and self._idle_connection_timeout > 0:
             self._idle_connections_collect_task = self._loop.create_task(
@@ -468,18 +472,11 @@ class ConnectionsPool(AbcPool):
                 parser=self._parser_class,
                 timeout=self._create_connection_timeout,
                 connection_cls=self._connection_cls,
+                post_init=self._connection_post_init,
             )
         except asyncio.TimeoutError:
             raise ConnectTimeoutError(address)
 
-        if self._readonly:
-            try:
-                await conn.set_readonly(self._readonly)
-            except (asyncio.CancelledError, Exception):
-                conn.close()
-                await conn.wait_closed()
-                raise
-
         conn.set_last_use_generation(self._idle_connections_collect_gen)
 
         return conn
@@ -592,6 +589,21 @@ class ConnectionsPool(AbcPool):
         await asyncio.sleep(time_split * 3 + jitter_time)
 
     async def _idle_connections_collector(self) -> None:
+        def is_need_close(current_gen: int, conn: AbcConnection) -> bool:
+            if conn.closed:
+                return True
+            if conn._waiters:
+                return False
+            if conn.in_pubsub:
+                return False
+            if conn.in_transaction:
+                return False
+
+            conn_gen = conn.get_last_use_generation()
+            if conn_gen == 0 or current_gen - conn_gen <= 1:
+                return False
+            return True
+
         close_waiters: Set[asyncio.Task] = set()
         while not self.closed:
             self._idle_connections_collect_running = False
@@ -604,28 +616,21 @@ class ConnectionsPool(AbcPool):
                 continue
 
             async with self._cond:
-                conn_iterator = itertools.chain(
-                    self._pool,
-                    [self._pubsub_conn, self._sharded_pubsub_conn],
-                )
-                for conn in conn_iterator:
+                pool_size = len(self._pool)
+                for conn in self._pool:
+                    if pool_size <= self._minsize:
+                        break
+                    if is_need_close(current_gen, conn):
+                        conn.close()
+                        close_waiters.add(asyncio.ensure_future(conn.wait_closed()))
+                        pool_size -= 1
+
+                if conn in (self._pubsub_conn, self._sharded_pubsub_conn):
                     if conn is None:
                         continue
-                    if conn.closed:
-                        continue
-                    if conn._waiters:
-                        continue
-                    if conn.in_pubsub:
-                        continue
-                    if conn.in_transaction:
-                        continue
-
-                    conn_gen = conn.get_last_use_generation()
-                    if conn_gen == 0 or current_gen - conn_gen <= 1:
-                        continue
-
-                    conn.close()
-                    close_waiters.add(asyncio.ensure_future(conn.wait_closed()))
+                    if is_need_close(current_gen, conn):
+                        conn.close()
+                        close_waiters.add(asyncio.ensure_future(conn.wait_closed()))
 
             if close_waiters:
                 await asyncio.wait(close_waiters)
